from django.conf.urls.defaults import patterns, url

from cbv_formpreview.tests.tests import TestFormPreview

urlpatterns = patterns('',
    url(r'^preview/', TestFormPreview.as_view(), name='preview')
)



from django import forms
from django.http import HttpResponse
from django.core.urlresolvers import reverse
from django.test import TestCase

from cbv_formpreview import preview

SUCCESS_STRING = "Done was called!"


class TestForm(forms.Form):
    field1 = forms.CharField()
    field1_ = forms.CharField()
    bool1 = forms.BooleanField(required=False)


class TestFormPreview(preview.FormPreview):
    form_class = TestForm
    form_template = 'cbv_formpreview/form.html'
    preview_template = 'cbv_formpreview/preview.html'

    def get_context_data(self, **kwargs):
        context = super(TestFormPreview, self).get_context_data(**kwargs)
        context['custom_context'] = True
        return context

    def get_initial(self):
        return {'field1': 'Works!'}

    def done(self, form):
        return HttpResponse(SUCCESS_STRING)


class PreviewTests(TestCase):
    def setUp(self):
        # Create a FormPreview instance to share between tests
        self.preview = TestFormPreview()
        input_template = '<input type="hidden" name="%s" value="%s" />'
        self.input = input_template % (self.preview.unused_name('stage'), "%d")
        self.test_data = {'field1': u'foo', 'field1_': u'bar'}
        self.url = reverse('preview')

    def test_unused_name(self):
        """
        Verifies name mangling to get uniue field name.
        """
        self.assertEqual(self.preview.unused_name('field1'), 'field1__')

    def test_form_get(self):
        """
        Test preview form retrieval.

        Use the client library to see if we can sucessfully retrieve
        the form (mostly testing the setup ROOT_URLCONF
        process). Verify that an additional  hidden input field
        is created to manage the stage.

        """
        response = self.client.get(self.url)
        stage = self.input % 1
        self.assertContains(response, stage, 1)
        self.assertEqual(response.context['custom_context'], True)
        self.assertEqual(response.context['form'].initial, {'field1': 'Works!'})

    def test_invalid_form(self):
        """ Verifies that an invalid form displays the correct template. """
        self.test_data.pop('field1')
        response = self.client.post(self.url, self.test_data)
        self.assertTemplateUsed(response, 'cbv_formpreview/form.html')

    def test_form_preview(self):
        """
        Test contrib.formtools.preview form preview rendering.

        Use the client library to POST to the form to see if a preview
        is returned.  If we do get a form back check that the hidden
        value is correctly managing the state of the form.

        """
        # Pass strings for form submittal and add stage variable to
        # show we previously saw first stage of the form.
        self.test_data.update({'stage': 1})
        response = self.client.post(self.url, self.test_data)
        # Check to confirm stage is set to 2 in output form.
        stage = self.input % 2
        self.assertContains(response, stage, 1)
        # Check that the correct context was passed to the template
        self.assertEqual(response.context['custom_context'], True)

    def test_form_submit(self):
        """
        Test contrib.formtools.preview form submittal.

        Use the client library to POST to the form with stage set to 3
        to see if our forms done() method is called. Check first
        without the security hash, verify failure, retry with security
        hash and verify sucess.

        """
        # Pass strings for form submittal and add stage variable to
        # show we previously saw first stage of the form.
        self.test_data.update({'stage':2})
        response = self.client.post(self.url, self.test_data)
        self.assertNotEqual(response.content, SUCCESS_STRING)
        form_hash = self.preview.security_hash(TestForm(self.test_data))
        self.test_data.update({'hash': form_hash})
        response = self.client.post(self.url, self.test_data)
        self.assertEqual(response.content, SUCCESS_STRING)

    def test_bool_submit(self):
        """
        Test contrib.formtools.preview form submittal when form contains:
        BooleanField(required=False)

        Ticket: #6209 - When an unchecked BooleanField is previewed, the preview
        form's hash would be computed with no value for ``bool1``. However, when
        the preview form is rendered, the unchecked hidden BooleanField would be
        rendered with the string value 'False'. So when the preview form is
        resubmitted, the hash would be computed with the value 'False' for
        ``bool1``. We need to make sure the hashes are the same in both cases.

        """
        self.test_data.update({'stage':2})
        form_hash = self.preview.security_hash(TestForm(self.test_data))
        self.test_data.update({'hash': form_hash, 'bool1': u'False'})
        response = self.client.post(self.url, self.test_data)
        self.assertEqual(response.content, SUCCESS_STRING)

    def test_form_submit_good_hash(self):
        """
        Test contrib.formtools.preview form submittal, using a correct
        hash
        """
        # Pass strings for form submittal and add stage variable to
        # show we previously saw first stage of the form.
        self.test_data.update({'stage':2})
        response = self.client.post(self.url, self.test_data)
        self.assertNotEqual(response.content, SUCCESS_STRING)

        form_hash = self.preview.security_hash(TestForm(self.test_data))
        self.test_data.update({'hash': form_hash})
        response = self.client.post(self.url, self.test_data)
        self.assertEqual(response.content, SUCCESS_STRING)

    def test_form_submit_bad_hash(self):
        """
        Test contrib.formtools.preview form submittal does not proceed
        if the hash is incorrect.
        """
        # Pass strings for form submittal and add stage variable to
        # show we previously saw first stage of the form.
        self.test_data.update({'stage':2})
        response = self.client.post(self.url, self.test_data)
        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.content, SUCCESS_STRING)

        self.test_data.update({'hash': 'the-wrong-hash'})
        response = self.client.post(self.url, self.test_data)
        self.assertTemplateUsed(response, 'cbv_formpreview/preview.html')
        self.assertNotEqual(response.content, SUCCESS_STRING)



#!/usr/bin/env python
import os
import sys

from django.conf import settings

if not settings.configured:
    databases = {'default': {'ENGINE': 'django.db.backends.sqlite3'}}
    settings.configure(
        DATABASES=databases,
        ROOT_URLCONF='cbv_formpreview.tests.urls',
        INSTALLED_APPS=('cbv_formpreview', 'cbv_formpreview.tests')
    )

from django.test.simple import DjangoTestSuiteRunner

def runtests(*test_args):
    parent = os.path.join(os.path.dirname(os.path.abspath(__file__)), '..', '..')
    sys.path.insert(0, parent)
    test_runner = DjangoTestSuiteRunner(verbosity=1, interactive=True)
    failures = test_runner.run_tests(test_args)
    sys.exit(failures)

if __name__ == '__main__':
    runtests(*sys.argv[1:])



import os
from setuptools import setup, find_packages

VERSION = '0.2.0a1'
README_FILENAME = 'README.rst'

readme = open(os.path.join(os.path.dirname(__file__), README_FILENAME))
long_description = readme.read()
readme.close()

setup(
    name='django-cbv-formpreview',
    version=VERSION,
    author='Ryan Kaskel',
    author_email='dev@ryankaskel.com',
    url='https://github.com/ryankask/django-cbv-formpreview',
    license='BSD',
    description='Django\'s FormPreview updated to use class based views.',
    long_description=long_description,
    packages=find_packages(),
    package_data = {
        'cbv_formpreview': ['templates/cbv_formpreview/*.html']
    },
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    test_suite='cbv_formpreview.tests.runtests.runtests'
)



from django.conf import settings
from django.utils.crypto import constant_time_compare
from django.contrib.formtools.utils import form_hmac
from django.views.generic import FormView

PREVIEW_STAGE = 'preview'
POST_STAGE = 'post'
STAGE_FIELD = 'stage'
HASH_FIELD = 'hash'


class FormPreview(FormView):
    preview_template = None
    form_template = None

    def dispatch(self, request, *args, **kwargs):
        self._preview_stage = PREVIEW_STAGE
        self._post_stage = POST_STAGE
        stages = {'1': self._preview_stage, '2': self._post_stage}

        posted_stage = request.POST.get(self.unused_name(STAGE_FIELD))
        self._stage = stages.get(posted_stage, self._preview_stage)

        return super(FormPreview, self).dispatch(request, *args, **kwargs)

    def unused_name(self, name):
        """ Given a first-choice name, adds an underscore to the name until it
        reaches a name that isn't claimed by any field in the form.

        This is calculated rather than being hard-coded so that no field names
        are off-limits for use in the form. """
        while 1:
            try:
                self.form_class.base_fields[name]
            except KeyError:
                break # This field name isn't being used by the form.
            name += '_'
        return name

    def get(self, request, *args, **kwargs):
        self.template_name = self.form_template
        return super(FormPreview, self).get(request, *args, **kwargs)

    def _check_security_hash(self, token, form):
        expected = self.security_hash(form)
        return constant_time_compare(token, expected)

    def form_valid(self, form):
        context = self.get_context_data(form=form)
        hash_field_name = self.unused_name(HASH_FIELD)

        if self._stage == self._preview_stage:
            self.process_preview(form, context)
            context['hash_field'] = hash_field_name
            context['hash_value'] = self.security_hash(form)
            self.template_name = self.preview_template
            return self.render_to_response(context)
        else:
            form_hash = self.request.POST.get(hash_field_name, '')
            if not self._check_security_hash(form_hash, form):
                return self.failed_hash(context) # Security hash failed.
            return self.done(form)

    def form_invalid(self, form):
        self.template_name = self.form_template
        return super(FormPreview, self).form_invalid(form)

    def get_context_data(self, **kwargs):
        kwargs.update({'stage_field': self.unused_name(STAGE_FIELD)})
        return kwargs

    def process_preview(self, form, context):
        """ Given a validated form, performs any extra processing before
        displaying the preview page, and saves any extra data in
        context. """
        pass

    def security_hash(self, form):
        """ Calculates the security hash for the given HttpRequest and
        Form instances.

        Subclasses may want to take into account request-specific information,
        such as the IP address. """
        return form_hmac(form)

    def failed_hash(self, context):
        """ Returns an HttpResponse in the case of an invalid security
        hash. """
        self._stage = self._preview_stage
        return self.post(self.request)

    def done(self, form):
        """ The only method required to be overriden by subclasses. Does
        something with the form and returns an HttpResponseRedirect. """
        msg = 'You must define a done() method on your %s subclass.'
        raise NotImplementedError(msg % self.__class__.__name__)
